/* $Id: bns_user.cc,v 1.2 2005/04/08 13:34:25 sds Exp $ */

/*
 * $Log: bns_user.cc,v $
 * Revision 1.2  2005/04/08 13:34:25  sds
 * minor change to comments. Added GNU license.
 *
 * Revision 1.1  2005/04/07 15:21:44  sds
 * Initial revision
 *
 */

/*
bns_user.cc
By Steve D. Sharples.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The author's email address is steve.sharples@nottingham.ac.uk
*/

/* This is bns_user.cc by Steve D Sharples (sds)
 *
 * It is a userspace library to provide useful functions for controlling the
 * BNS 512x512 SLM. It his hacked from the (now deprecated) "bns_main"
 * library, which did the same thing plus other "laser ultrasound-specific"
 * functions, such as producing images of arcs. Those functions
 * are now in the "bns_local" library, which is now part of the /source/hardware/ 
 * tree.
 *
 * Note that as the code currently stands, only one card is supported.
 */

#include "bns_user.h"

/* BNS file descriptor. Note only one card can be used at once */
static int BNS_FD = BNS_NOT_CONNECTED;
int	*dummy=NULL;



/* Simple wrapper function for opening the BNS device*/
int	bns_open(void){

	/* if we've already opened the device, just return */
	if(BNS_FD!=BNS_NOT_CONNECTED) return BNS_OK;

	/* otherwise open the device */
	BNS_FD=open(BNS_SLM,O_RDWR);
	if(BNS_FD<0){
		printf("BNS ERROR: Failed to open device %d (installed?)\n",BNS_FD);
		return BNS_CANT_OPEN_DEVICE;
		}
	return BNS_OK;
	}

/* Simple wrapper function */
void	bns_disable_writes(void){
	#ifndef BNS_ALLOW_USER_DPEN_CONTROL
	usleep(BNS_DPEN_PARANOID_PAUSE);
	ioctl(BNS_FD,BNS_DISABLE_WRITE,dummy);
	usleep(BNS_DPEN_PARANOID_PAUSE);
	#endif
	}

/* Power on */
int	bns_power_on(void){
int	ret;
	ret=ioctl(BNS_FD,BNS_POWER_ON,dummy);
	/* Note we need to disable writes again (#ifndef BNS_ALLOW_USER_DPEN_CONTROL)
	 * because writes are enabled at the end of the bns_power_on() fn (WITHIN THE
	 * slm.c DRIVER... ie different function, same name)... if we don't do this then
	 * images will not be updated
	 */
	bns_disable_writes();
	}

/* Power off */
int	bns_power_off(void){
	return ioctl(BNS_FD,BNS_POWER_OFF,dummy);
	}

/* Set the time period */
int	bns_set_period(float time_period){
	/* Conversion from seconds to SLM time units, where one "unit" is equal
	 * to:    / 1/1015 \
	 *       |  ------  | seconds
	 *        \   6    /
	 */
	return ioctl(BNS_FD,BNS_SET_PERIODS,(int)(time_period/164.4e-6));
	}

/* Set laser A time and amplitude */
int	bns_set_laser_a(unsigned int t_period, unsigned int amplitude){
	div_t l_div;
	l_div=div((int)t_period,16);
	return ioctl(BNS_FD,BNS_SET_LASERA,(unsigned int)(65536*l_div.quot)+(unsigned int)(256*l_div.rem)+(unsigned int)amplitude);
	}

/* Set laser B time and amplitude */
int	bns_set_laser_b(unsigned int t_period, unsigned int amplitude){
	div_t l_div;
	l_div=div((int)t_period,16);
	return ioctl(BNS_FD,BNS_SET_LASERB,(unsigned int)(65536*l_div.quot)+(unsigned int)(256*l_div.rem)+(unsigned int)amplitude);
	}

/* Loads an image file from disk.
 * Option to print diagnostics to the screen. */
int	bns_load_image_file(char *filename, unsigned char *image, BOOL diag){
FILE	*fi;

	/* load the file */
	fi=fopen(filename,"r");
	if(fi>0){
		if(diag) printf("Loading BNS image file \"%s\"\n",filename);
		fread(image,sizeof(unsigned char),BNS_SIZE,fi);
		fclose(fi);
		}
	else{
		printf("BNS ERROR: could not load BNS image file \"%s\"\n",filename);
		return BNS_CANT_OPEN_IMAGE_FILE;
		}

	return BNS_OK;
	}

/* Saves a bns image file to disk.
 * Option to print diagnostics */
int	bns_save_image_file(char *filename, unsigned char *image, BOOL diag){
FILE	*fo;

	/* save the file */
	fo=fopen(filename,"w");
	if(fo>0){
		if(diag) printf("Saving BNS image file \"%s\"\n",filename);
		fwrite((void *)image,sizeof(unsigned char),BNS_SIZE,fo);
		fclose(fo);
		}
	else{
		printf("BNS ERROR: could not save BNS image file \"%s\"\n",filename);
		return BNS_CANT_SAVE_IMAGE_FILE;
		}

	return BNS_OK;
	}

/* Uploads an image (stored in the image[BNS_SIZE] array)
 * to a frame of your choice in the SLM buffer.
 * Option to display the frame after you've uploaded it. 
 * Option to print diagnostics to the screen. */
int	bns_upload_frame(unsigned char *image, int frame, BOOL display_frame, BOOL diag){
int	err,pos,y;

	#ifdef	BNS_ALLOW_USER_DPEN_CONTROL
	ioctl(BNS_FD,BNS_ENABLE_WRITE,dummy);
	#endif

	/* tell the card that we're about to upload */
	ioctl(BNS_FD,BNS_UPLOAD_IMAGE,frame);
	err=BNS_ROW_SIZE;
	for(pos=0,y=0;y<BNS_N_ROWS;y++,pos+=BNS_ROW_SIZE){ /* load a row at a time */
		err=write(BNS_FD,image+pos,BNS_ROW_SIZE);
		if(err!=BNS_ROW_SIZE){ /* uh-oh, trouble at t'mill */
			printf("BNS ERROR: WRITE ERROR (writes enabled?) err=%d\n",err);
			return BNS_IMAGE_WRITE_ERROR;
			}
		}
	/* note we don't enable writes again, in case we want to upload another image... */

	if(display_frame) bns_display_frame(frame, diag);
	else	if(diag) printf("New image in frame %d is NOT being displayed\n",frame);

	return BNS_OK;
	}

/* Display a numbered frame in the SLM image buffer. 
 * Option to print diagnostics to the screen. */
int	bns_display_frame(int frame, BOOL diag){

	if(diag) printf("Displaying frame %d\n",frame);
	#ifdef	BNS_ALLOW_USER_DPEN_CONTROL
	ioctl(BNS_FD,BNS_DISABLE_WRITE,dummy);
	#endif
	ioctl(BNS_FD,BNS_DISPLAY_IMAGE,frame);

	return BNS_OK;
	}

/* Simple wrapper function to perform the common task of loading
 * an image file from disk, and uploading it to the SLM buffer.
 * Option to display the frame after you've uploaded it. 
 * Option to print diagnostics to the screen. */
int	bns_load_and_upload_frame(char *filename, int frame, BOOL display_frame, BOOL diag){
int	ret;
unsigned char	image[BNS_SIZE];

	ret=bns_load_image_file(filename, image, diag);
	if(ret!=BNS_OK) return ret;

	bns_upload_frame(image, frame, display_frame, diag);
	if(ret!=BNS_OK) return ret;

	return BNS_OK;
	}

