/*
bns_laser_ab.c
By Steve D. Sharples.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The author's email address is steve.sharples@nottingham.ac.uk
*/

#include "../../library/bns_user.h"

/*BOOL is #defined in bns_user.h (as are TRUE and FALSE) */
BOOL	sc(char *, char *);

main(int argc,char ** argv){
float		time_period;
unsigned int	lasera_amp;
unsigned int	laserb_amp;
unsigned int	lasera_time;
unsigned int	laserb_time;

int		index=1;
BOOL		verbose_mode=FALSE;
BOOL		got_lasera_amp=FALSE;
BOOL		got_laserb_amp=FALSE;
BOOL		got_lasera_time=FALSE;
BOOL		got_laserb_time=FALSE;
BOOL		got_time_period=FALSE;

	if((argc==1)||(sc(argv[1],"-?"))){
		printf("Need some arguments!\n\n");
		printf("-aa  -a_amp       : Laser A amplitude (0->255)\n");
		printf("-at  -a_time      : Laser A time      (0->65535)\n");
		printf("-ba  -b_amp       : Laser B amplitude (0->255)\n");
		printf("-bt  -b_time      : Laser B time      (0->65535)\n");
		printf("-t   -time_period : time period in secs (not set by default)\n");
		printf("-v   -verbose     : confirm what's been done\n");
		exit(3);
		}

	while(index<argc){
		if(sc(argv[index],"-v")||sc(argv[index],"-verbose")){
			verbose_mode=TRUE;
			}
		if(sc(argv[index],"-aa")||sc(argv[index],"-a_amp")){
			sscanf(argv[++index],"%u",&lasera_amp);
			got_lasera_amp=TRUE;
			}
		if(sc(argv[index],"-at")||sc(argv[index],"-a_time")){
			sscanf(argv[++index],"%u",&lasera_time);
			got_lasera_time=TRUE;
			}
		if(sc(argv[index],"-ba")||sc(argv[index],"-b_amp")){
			sscanf(argv[++index],"%u",&laserb_amp);
			got_laserb_amp=TRUE;
			}
		if(sc(argv[index],"-bt")||sc(argv[index],"-b_time")){
			sscanf(argv[++index],"%u",&laserb_time);
			got_laserb_time=TRUE;
			}
		if(sc(argv[index],"-t")||sc(argv[index],"-time_period")){
			sscanf(argv[++index],"%f",&time_period);
			got_time_period=TRUE;
			}
		index++;
		}

	/* Check the values that we've been passed. They should all be within a certain range.
	 * Only check the values that we've been passed, though. We are only going to change 
	 * parameters for which values have been passed. */
	if(got_lasera_amp==TRUE && (lasera_amp<0 || lasera_amp>255)){printf("ERROR! Laser A amp should be >=0 and <=255! Quitting...\n");exit(4);}
	if(got_laserb_amp==TRUE && (laserb_amp<0 || laserb_amp>255)){printf("ERROR! Laser B amp should be >=0 and <=255! Quitting...\n");exit(4);}
	if(got_lasera_time==TRUE && (lasera_time<0 || lasera_time>65535)){printf("ERROR! Laser A time should be >=0 and <=65535! Quitting...\n");exit(4);}
	if(got_laserb_time==TRUE && (laserb_time<0 || laserb_time>65535)){printf("ERROR! Laser B time should be >=0 and <=65535! Quitting...\n");exit(4);}
	if(got_time_period==TRUE && ((int)(time_period/164.4e-6)<6 || time_period>10)){printf("ERROR! Time period should be >=0.001 and <=10! Quitting...\n");exit(4);}
	/* If setting laser A or laser B properties, both time AND amp need to be stated. */
	if(got_lasera_amp!=got_lasera_time){printf("ERROR! If you're setting Laser A parameters, you need to set both amp AND time.\nQuitting...\n");exit(4);}
	if(got_laserb_amp!=got_laserb_time){printf("ERROR! If you're setting Laser B parameters, you need to set both amp AND time.\nQuitting...\n");exit(4);}

	if(verbose_mode==TRUE){
		printf("\nbns_laser_ab: verbose mode selected. New values:\n");
		if(got_time_period==TRUE) printf("Time period: %f\n",time_period);
		if(got_lasera_amp==TRUE) printf("Laser A amp: %u\tLaser A time: %u\n",lasera_amp,lasera_time);
		if(got_laserb_amp==TRUE) printf("Laser B amp: %u\tLaser B time: %u\n",laserb_amp,laserb_time);
		}

	if(bns_open()!=BNS_OK) exit(2);

	if(got_time_period==TRUE) bns_set_period(time_period);
	if(got_lasera_amp==TRUE) bns_set_laser_a(lasera_time,lasera_amp);
	if(got_laserb_amp==TRUE) bns_set_laser_b(laserb_time,laserb_amp);

	if(verbose_mode==TRUE) printf("Done.\n");
	}

BOOL sc(char *con,char *var){
	if(strcmp(con,var)==0){
		return TRUE;
		}
	return FALSE;
	}
