/*
bns_upload_frame.c
By Steve D. Sharples.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The author's email address is steve.sharples@nottingham.ac.uk
*/

#include "../../library/bns_user.h"

#define	DEFAULT_FRAME	0

/*BOOL is #defined in bns_user.h (as are TRUE and FALSE) */
BOOL	sc(char *, char *);
void	print_help(void);

main(int argc,char ** argv){

unsigned char	image[BNS_SIZE];
int		index		= 1;
char		*filename;
BOOL		got_filename	= FALSE;
float		period;
BOOL		set_period	= FALSE;
int		frame		= DEFAULT_FRAME;
BOOL		power_on	= FALSE;
BOOL		verbose_mode	= FALSE;
FILE		*fi;

	if(argc==1){
		printf("Need some arguments!\n\n");
		print_help();
		exit(1);
		}

	while(index<argc){
                if(sc(argv[index],"-h")||sc(argv[index],"-help")||sc(argv[index],"-?")){
                        print_help();
                        exit(1);
                        }
		if(sc(argv[index],"-v")||sc(argv[index],"-verbose")){
			verbose_mode=TRUE;
			}
		if(sc(argv[index],"-n")||sc(argv[index],"-no_power")){
			power_on=FALSE;
			}
		if(sc(argv[index],"-p")||sc(argv[index],"-power")){
			power_on=TRUE;
			}
		if(sc(argv[index],"-f")||sc(argv[index],"-filename")){
                        filename=argv[++index];
			got_filename=TRUE;
			}
		if(sc(argv[index],"-t")||sc(argv[index],"-time_period")){
			sscanf(argv[++index],"%f",&period);
			set_period=TRUE;
			}
		if(sc(argv[index],"-fr")||sc(argv[index],"-frame")){
			sscanf(argv[++index],"%d",&frame);
			}
		index++;
		}

	if(got_filename==FALSE){
		printf("ERROR: you must provide a filename!\n\n");
		print_help();
		exit(1);
		}

	if(bns_open()!=BNS_OK) exit(2);

	if(power_on==TRUE) bns_power_on();

	if(verbose_mode==TRUE){
		printf("Uploading \"%s\" to frame %d, power has ",filename,frame);
        	if(power_on==FALSE) printf("NOT");
		printf(" been turned on");
		}

	if(set_period==TRUE){
		if(verbose_mode==TRUE){
			printf(", period: %fs",period);
			}
		bns_set_period(period);
		}

	if(verbose_mode==TRUE){
		printf("\n");
		}

	if(bns_load_and_upload_frame(filename, frame, TRUE, FALSE)!=BNS_OK) exit(2); // display_frame=TRUE, diag=FALSE

	return 0;
	}	

BOOL	sc(char *con,char *var){
	if(strcmp(con,var)==0){
		return TRUE;
		}
	return FALSE;
        }

void	print_help(void){
	printf("Help information for \"bns_upload_frame\"\n");
	printf("bns_upload_frame is a simple utility for loading an image from disk\n");
	printf("(it should be a 512x512 array of unsigned ints, and will probably have\n");
	printf("a \".bns\" extension), upload this image to a frame of your choice\n");
	printf("on the BNS SLM buffer, and then display the frame. You can\n");
	printf("optionally set the time period while you're at it, and choose\n");
	printf("whether or not you want to turn on the SLM power.\n\n");

	printf("REQUIRED ARGUMENTS:\n");
	printf("-f   -filename    : filename of image (should have .bns extension)\n\n");
	printf("OPTIONAL ARGUMENTS:\n");
	printf("-fr  -frame       : frame to upload image to  default %d\n",DEFAULT_FRAME);
	printf("-t   -time_period : time period (in seconds)  (not set by default)\n");
	printf("-n   -no_power    : do NOT turn on SLM power  (default)\n");
	printf("-p   -power       : DO turn on SLM power      (NOT default)\n");
	printf("-v   -verbose     : confirm what's been done  (not default)\n");
	}

