%MAP=IMAGE2BNS(IMAGE_FILENAME,IMAGE_FORMAT,BNS_FILENAME)
%
%Converts an image file to a format suitable for uploading
%to the BNS SLM, ie a 512x512 array of unsigned integers.
%It uses the built-in Matlab command "IMREAD"... type
%HELP IMREAD to find out information about the IMAGE_FORMAT
%argument. Note only grayscale files can be converted. The
%image is rescaled to the correct bit depth for the BNS SLM
%(8 bits).
%
%Note that your image must be 512x512 pixels... if it is not,
%the function will put your image into the MAP array, but
%will not save the image to disk as a ".bns" file.
%
%If you do not supply BNS_FILENAME, the function will attempt
%to choose a sensible name for you, which will be the
%IMAGE_FILENAME with the letters after the last "." replaced
%by "bns" (eg "my_image.gif" becomes "my_image.bns")
%
%See also MAP2BNS, BNS2MAP.

function map=image2bns(image_filename,image_format,bns_filename)

%image2bns.m
%By Steve D. Sharples.

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

%You should have received a copy of the GNU General Public License
%along with this program; if not, write to the Free Software
%Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

%The author's email address is steve.sharples@nottingham.ac.uk

map=imread(image_filename,image_format);
image_info=imfinfo(image_filename,image_format);

if strcmp(getfield(image_info,'ColorType'),'grayscale')==0
	disp('Error! I can only successfully convert greyscale images.');
	return;
	end;

if (size(map,1)~=512 | size(map,2)~=512)
	disp('Error! I have converted your image file to a map, but')
	disp('it is not of the correct size (512x512)')
	return;
	end;

max_value=getfield(image_info,'MaxSampleValue');
% Need to rescale the array.
map=(map./max_value).*255;

% If we don't specify the bns filename, we try and make a sensible
% one out of the image filename.
if exist('bns_filename')==0
	tmp=strfind(image_filename,'.');% all the dots
	dot_pos=tmp(length(tmp));	% ie last dot
	if isempty(dot_pos)
		bns_filename=strcat(image_filename,'.bns');
	else
		bns_filename=strcat(image_filename(1:dot_pos),'bns');
		end;
	end;

map2bns(map,bns_filename);

